function [h S] = MVSVRW(Ystar,h,iSig,iVh)

% This function simulates log-volatilities for a multivariate stochastic
% volatility model with independent random-walk transitions.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

n = size(iSig,1);
Tn = size(h,1);
%% normal mixture
pi = [0.0073 .10556 .00002 .04395 .34001 .24566 .2575];
mi = [-10.12999 -3.97281 -8.56686 2.77786 .61942 1.79518 -1.08819] - 1.2704;  %% means already adjusted!! %%
sigi = [5.79596 2.61369 5.17950 .16735 .64009 .34023 1.26261];
sqrtsigi = sqrt(sigi);

%% sample S from a 7-point distrete distribution
temprand = rand(Tn,1);
q = repmat(pi,Tn,1).*normpdf(repmat(Ystar,1,7),repmat(h,1,7)+repmat(mi,Tn,1), repmat(sqrtsigi,Tn,1));
q = q./repmat(sum(q,2),1,7);
S = 7 - sum(repmat(temprand,1,7)<cumsum(q,2),2)+1;
    
%% sample h
Hh =  spdiags( -ones( Tn, 1 ), -n, speye( Tn ) );
invSh = blkdiag( iVh, kron( speye( Tn / n - 1 ), iSig ) );
dconst = mi(S)'; invOmega = spdiags(1./sigi(S)',0,Tn,Tn);
Kh = Hh'*invSh*Hh;
Ph = Kh + invOmega;
Ch = chol(Ph);
hhat = Ph\(invOmega*(Ystar-dconst));
h = hhat + Ch\randn(Tn,1);