function out = igrand( theta, chi )

% This function generates random samples from the inverse-Gaussian
% distribution with parameters theta and chi.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

n = size( theta );

if ( n ~= size( chi ) )
	error 'Parameter matrices must be the same size.';
end

chisq1 = randn( n ) .^ 2;

out = theta + 0.5 * theta ./ chi .* ( theta .* chisq1 - sqrt( 4 * theta .* chi .* chisq1 + theta .^ 2 .* chisq1 .^ 2 ) );

l = rand( n ) >= theta ./ ( theta + out );
out( l ) = theta( l ) .^ 2 ./ out( l );
